"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable no-await-in-loop */
const path_1 = __importDefault(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const glob_1 = __importDefault(require("glob"));
const vue_docgen_api_1 = require("vue-docgen-api");
const chalk_1 = __importDefault(require("chalk"));
const print_1 = __importDefault(require("./utils/print"));
const templates_1 = __importDefault(require("./templates"));
const parse_interface_1 = __importDefault(require("./utils/parse-interface"));
const utils_1 = require("./utils");
const parse_material_1 = __importDefault(require("./utils/parse-material"));
const slot_tag_handler_1 = require("./slot-tag-handler");
const MD_TEMPLATE = 'TEMPLATE.md';
const MD_TARGET = 'README.zh-CN.md';
const MD_TARGET_EN = 'README.en-US.md';
const MD_MATERIAL_README = 'docs/README.md';
const MD_MATERIAL_README_EN = 'docs/README.en-US.md';
const TEMPLATE_GLOB = `components/*/${MD_TEMPLATE}`;
const getApiTmpl = (componentDoc, type, lang) => {
    const componentDocList = Array.isArray(componentDoc)
        ? componentDoc
        : [componentDoc];
    const res = [];
    componentDocList.forEach((item) => {
        const { displayName, props, events, methods, slots, tags } = item;
        const getTmpl = (suffix, content) => {
            var _a, _b;
            if (!content)
                return '';
            let title = displayName;
            if (type === 'component') {
                title = (tags === null || tags === void 0 ? void 0 : tags.noBrackets)
                    ? displayName
                    : `<${(0, utils_1.toKebabCase)(displayName)}>`;
                title = `\`${title}\` ${suffix}`;
            }
            if (tags === null || tags === void 0 ? void 0 : tags.version) {
                const version = (_a = tags.version[0]) === null || _a === void 0 ? void 0 : _a.description;
                version && (title += ` (${version})`);
            }
            let description = '';
            if (suffix === 'Props' && (tags === null || tags === void 0 ? void 0 : tags[lang])) {
                description = (_b = tags[lang][0]) === null || _b === void 0 ? void 0 : _b.description;
            }
            return `### ${title}${description ? `\n${description}` : ''}\n${content}`;
        };
        const propsTmpl = getTmpl('Props', templates_1.default.props(props || [], { isInterface: type === 'interface' }, lang));
        const eventsTmpl = getTmpl('Events', templates_1.default.events(events || [], lang));
        const methodsTmpl = getTmpl('Methods', templates_1.default.methods(methods || [], lang));
        const slotsTmpl = getTmpl('Slots', templates_1.default.slots(slots || [], lang));
        res.push(`\n${propsTmpl}${eventsTmpl}${methodsTmpl}${slotsTmpl}\n`);
    });
    return res.join('\n');
};
const getComponentNameFormDir = (dir) => {
    const matchRes = dir.match(/[^/]+$/g);
    return matchRes ? matchRes[0] : '';
};
// 占位符替换
const replacePlaceholderToDoc = ({ dir, template, placeholderMatchers, parser, type, lang, }) => __awaiter(void 0, void 0, void 0, function* () {
    let result = template;
    const matches = Array.from(result.matchAll(placeholderMatchers));
    for (const item of matches) {
        try {
            // @ts-ignore
            // eslint-disable-next-line
            const componentDoc = yield parser(path_1.default.resolve(dir, item[1]), {
                addScriptHandlers: [slot_tag_handler_1.slotTagHandler],
            });
            result = result.replace(item[0], getApiTmpl(componentDoc, type, lang));
        }
        catch (err) {
            // eslint-disable-next-line no-console
            console.log(err, dir);
        }
    }
    return result;
});
const docgen = ({ input, components, }) => __awaiter(void 0, void 0, void 0, function* () {
    const files = [];
    if (input) {
        const filename = path_1.default.resolve(process.cwd(), input);
        try {
            yield fs_extra_1.default.ensureFile(filename);
            files.push(filename);
        }
        catch (err) {
            return;
        }
    }
    else {
        // 读取组件列表
        const templates = glob_1.default.sync(TEMPLATE_GLOB, { absolute: true });
        if (components && components.length > 0) {
            const reg = new RegExp(components.join('|'));
            for (const template of templates) {
                if (reg.test(template)) {
                    files.push(template);
                }
            }
        }
        else {
            files.push(...templates);
        }
    }
    print_1.default.info('Start to generate document...');
    for (const filename of files) {
        const dirname = path_1.default.dirname(filename);
        const result = fs_extra_1.default.readFileSync(filename, 'utf8');
        let zhResult = (0, utils_1.getTemplate)(result, 'zh');
        let enResult = (0, utils_1.getTemplate)(result, 'en');
        // API占位符替换
        zhResult = yield replacePlaceholderToDoc({
            dir: dirname,
            template: zhResult,
            placeholderMatchers: /%%API\((.+?)\)%%/g,
            parser: vue_docgen_api_1.parse,
            type: 'component',
            lang: 'zh',
        });
        enResult = yield replacePlaceholderToDoc({
            dir: dirname,
            template: enResult,
            placeholderMatchers: /%%API\((.+?)\)%%/g,
            parser: vue_docgen_api_1.parse,
            type: 'component',
            lang: 'en',
        });
        // INTERFACE占位符替换
        zhResult = yield replacePlaceholderToDoc({
            dir: dirname,
            template: zhResult,
            placeholderMatchers: /%%INTERFACE\((.+?)\)%%/g,
            parser: parse_interface_1.default,
            type: 'interface',
            lang: 'zh',
        });
        enResult = yield replacePlaceholderToDoc({
            dir: dirname,
            template: enResult,
            placeholderMatchers: /%%INTERFACE\((.+?)\)%%/g,
            parser: parse_interface_1.default,
            type: 'interface',
            lang: 'en',
        });
        enResult = enResult.replace(/```yaml\n.+?```\n/s, (match) => `${match}\n*Auto translate by google.*\n`);
        zhResult = yield (0, parse_material_1.default)(zhResult, {
            matcher: /%%MATERIAL\((.+?)\)%%/,
            dirname,
        });
        enResult = yield (0, parse_material_1.default)(enResult, {
            matcher: /%%MATERIAL\((.+?)\)%%/,
            dirname,
        });
        try {
            const outputPath = input
                ? path_1.default.resolve(process.cwd(), MD_MATERIAL_README)
                : path_1.default.resolve(dirname, MD_TARGET);
            yield fs_extra_1.default.outputFile(outputPath, zhResult);
            const outputPath2 = input
                ? path_1.default.resolve(process.cwd(), MD_MATERIAL_README_EN)
                : path_1.default.resolve(dirname, MD_TARGET_EN);
            yield fs_extra_1.default.outputFile(outputPath2, enResult);
            if (!input) {
                const componentName = getComponentNameFormDir(dirname);
                print_1.default.success(`${chalk_1.default.black.bgGreen.bold(`Generate README of component ${componentName} Success!`)}`);
            }
        }
        catch (err) {
            print_1.default.error(err);
        }
    }
});
exports.default = docgen;
//# sourceMappingURL=index.js.map